MERGE_ARTICLES_FROM_DIFFERENT_SOURCES = """
I'm sending you a set of articles, potentially from multiple sources.
A source is the website where an article originates.
Some articles may be duplicates across different sources.
For these duplicated articles, please select one to be the primary article and list the others as its related articles.
"""
